using ArthaChitra.Model.Chart;
using ArthaChitra.Model.Common;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Text.Json;
using System.Text.Json.Serialization;
using System.Threading;
using System.Threading.Tasks;
using System.Xml.Serialization;
using System.Windows.Media;
using System.Windows.Input;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Net.Http;
using System.Windows.Baml2006;

namespace ArthaChitra.Model.SharpScript
{
    public sealed class GeminiAI : Indicator
    {
        IMenuItem menu = null;
        
        public GeminiAI() 
        {
    
        }

        protected override void OnBarUpdate()
        {
            
        }
        
        private async void OnClick(Window window)
        {
            if (string.IsNullOrEmpty(this.ApiKey))
                return;
                
			using (System.IO.MemoryStream ms = new System.IO.MemoryStream())
			{
			    ArthaChitra.Core.ViewModel.ChartViewModel.CaptureScreenshot(window, ms, new PngBitmapEncoder(), 0.0d, 0.0d);
            
                var question = new StringBuilder();
                question.Append($"Please do a technical analysis of the chart. The chart is of {this.Parent.Instrument.DisplayName} {this.Parent.Instrument.InstrumentType} as listed in {this.Parent.Instrument.Exchange} with periodicity of {this.Parent.DisplayName}.");
                if (this.Parent.Indicators.Length > 1)
                {
                    question.Append(" The following indicators are overlayed on the chart : ");
                    foreach (var indicator in this.Parent.Indicators)
                    {
                        if (indicator == null || indicator.State == State.Finalize || indicator.GetType() == typeof(GeminiAI))
                            continue;
                        question.Append($"{indicator.DisplayName}, ");
                    }
                }
			
				var payload = new GeminiPayload()
				{
				     contents = new GeminiContent[]
				     {
				         new GeminiContent()
				         {
				             parts = new Part[]
			             
				             {
				                 new Part()
				                 {
				                     text = question.ToString()
				                 },
				                 new Part()
				                 {
				                     inlineData = new Inlinedata()
				                     {
				                         mimeType = "image/png",
				                         data = Convert.ToBase64String(ms.ToArray())
				                     }
				                 }
				             }
				         }
				     }
				};
			
				var json = payload.GetJsonString();
			
				using (var client = new HttpClient() { Timeout = TimeSpan.FromSeconds(30)})
				{
				     //var req = new HttpRequestMessage(HttpMethod.Post, $"https://generativelanguage.googleapis.com/v1beta/models/gemini-1.5-flash-latest:generateContent?key={this.ApiKey}");
	                 var req = new HttpRequestMessage(HttpMethod.Post, $"https://generativelanguage.googleapis.com/v1beta/models/gemini-2.0-flash:generateContent?key={this.ApiKey}");
	                 req.Content = new StringContent(json, Encoding.UTF8, "application/json");
                 
	                 Log(Priority.Low, "Sending http req....");
	                 var response = await client.SendAsync(req);
	                 if (response == null)
	                    return;
                    
	                 var txt = await response.Content.ReadAsStringAsync();
	                 if (string.IsNullOrEmpty(txt))
	                    return;
                    
	                 var options = new JsonSerializerOptions
					 {
					    DefaultIgnoreCondition = JsonIgnoreCondition.WhenWritingNull,
					    PropertyNameCaseInsensitive = true,
					    ReadCommentHandling = JsonCommentHandling.Skip
					 };

					 var obj = JsonSerializer.Deserialize<GeminiResponse>(txt, options);
                
	                 if (obj == null || obj.candidates == null)
	                    return;
                    
	                StringBuilder sb = new();
	                foreach (var item in obj.candidates)
					{
					    foreach (var part in item.content.parts)
					    {
					        if (part == null)
					            continue;
				            
					        sb.AppendLine(part.text);
					    }
					}
				
					var str = sb.ToString();
					Log(str);
				
					this.MessageBox.ShowView(window, str);
				}
			}
			
        }

        protected override void OnStateChange()
        {
            switch (this.State)
            {
                case State.Initialize:
                    
                    break;
                case State.Configure:
                    break;
                case State.StartUp:
                    this.menu = this.AddContextMenu("Ask Gemini", new RelayCommand<Window>(OnClick), styleTemplate: Model.Common.ResourceKey.MenuItemWithParamWindowStyle);
                
                    break;
                case State.Historical:
                    break;
                case State.Realtime:
                    break;
                case State.Finalize:
                    this.TryRemoveContextMenuItem(this.menu);
                    break;
                default:
                    break;
            }
        }

        public string ApiKey { get; set; }

        public override object Template => $"{this.GetType().Name}Indicator";
   
    }

    public class GeminiPayload
	{
	    public GeminiContent[] contents { get; set; }
	    public string cachedContent { get; set; }
	    public GeminiContent systemInstruction { get; set; }
	    public Tool[] tools { get; set; }
	    public Safetysetting[] safetySettings { get; set; }
		public Generationconfig generationConfig { get; set; }
		public Labels labels { get; set; }
		
		public virtual string GetJsonString()
		{
		    var options = new JsonSerializerOptions
		    {
		        WriteIndented = true,
		        DefaultIgnoreCondition = JsonIgnoreCondition.WhenWritingNull,
		        Converters =
			    {
			        new JsonStringEnumConverter(JsonNamingPolicy.CamelCase)
			    }
		    };

		    return JsonSerializer.Serialize(this, this.GetType(), options);
		}
	}
	
	public class Tool
	{
	    public Functiondeclaration[] functionDeclarations { get; set; }
	}

	public class Functiondeclaration
	{
	    public string name { get; set; }
	    public string description { get; set; }
	    public object parameters { get; set; }
	}

    
	public class Safetysetting
	{
	    public HarmCategory category { get; set; }
	    public HarmBlockThreshold threshold { get; set; }
	}
	
	public enum HarmCategory
	{
	    HARM_CATEGORY_SEXUALLY_EXPLICIT,
	    HARM_CATEGORY_HATE_SPEECH,
	    HARM_CATEGORY_HARASSMENT,
        HARM_CATEGORY_DANGEROUS_CONTENT
	}
	
	public enum HarmBlockThreshold
	{
	    OFF,
	    BLOCK_NONE,
	    BLOCK_LOW_AND_ABOVE,
	    BLOCK_MEDIUM_AND_ABOVE,
	    BLOCK_ONLY_HIGH
	}
	
    public class Generationconfig
	{
	    public double temperature { get; set; }
	    public double topP { get; set; }
	    public double topK { get; set; }
	    public int candidateCount { get; set; }
	    public int maxOutputTokens { get; set; }
	    public float presencePenalty { get; set; }
	    public float frequencyPenalty { get; set; }
	    public string[] stopSequences { get; set; }
	    public string responseMimeType { get; set; }
	    public object responseSchema { get; set; }
	    public int seed { get; set; }
	    public bool responseLogprobs { get; set; }
	    public int logprobs { get; set; }
	    public bool audioTimestamp { get; set; }
	}

    public class Labels
	{
	    public string _string { get; set; }
	}

	public class GeminiContent
	{
	    public Part[] parts { get; set; }
	    public string role { get; set; }
	}

	public class Part
	{
	    public string text { get; set; }
	    public Inlinedata inlineData { get; set; }
	    public Filedata fileData { get; set; }
        public Videometadata videoMetadata { get; set; }
    }   

	public class Inlinedata
	{
	    public string mimeType { get; set; }
	    public string data { get; set; }
	}
	
	public class Filedata
	{
	    public string mimeType { get; set; }
	    public string fileUri { get; set; }
	}
	
	public class Videometadata
	{
	   public Offset startOffset { get; set; }
	   public Offset endOffset { get; set; }
	}
	
	public class Offset
	{
	    public int seconds { get; set; }
	    public int nanos { get; set; }
	}
	
	
	//Response
	public class GeminiResponse
	{
	    public Candidate[] candidates { get; set; }
	    public Usagemetadata usageMetadata { get; set; }
	    public string modelVersion { get; set; }
	}

	public class Usagemetadata
	{
	    public int promptTokenCount { get; set; }
	    public int candidatesTokenCount { get; set; }
	    public int totalTokenCount { get; set; }
	}

	public class Candidate
	{
	    public GeminiContent content { get; set; }
	    public string finishReason { get; set; }
	    public Citationmetadata citationMetadata { get; set; }
	    public float avgLogprobs { get; set; }
	}

	
	public class Citationmetadata
	{
	    public Citationsource[] citationSources { get; set; }
	}

	public class Citationsource
	{
	    public int startIndex { get; set; }
	    public int endIndex { get; set; }
	    public string uri { get; set; }
	}
	
}
